# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class BootstrapException(Exception):
    """
    Any exception from this package
    """
    pass


class BootstrapError(BootstrapException):
    """
    Any exception that is an error
    """
    pass
